﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Silverlight.Drawing;
using Infragistics.Silverlight.Timeline;
using System.Collections.Generic;

namespace SilverlightApplication1
{
    /// <summary>
    /// Represents a tool that is used moving the event points.
    /// </summary>
    public class EventPointMovingTool : Tool
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="EventPointMovingTool"/> class.
        /// </summary>
        /// <param name="timeline">The timeline.</param>
        public EventPointMovingTool(XamWebTimeline timeline)
            : base(timeline)
        {
            this.Timeline = timeline;
        }

        private XamWebTimeline _timeline;
        private XamWebTimeline Timeline
        {
            get { return _timeline; }
            set { _timeline = value; }
        }

        private EventPoint _handledPoint;
        private EventPoint HandledPoint
        {
            get { return _handledPoint; }
            set { _handledPoint = value; }
        }       

        private DateTimeAxis Axis
        {
            get
            {
                return this.Timeline.Axis as DateTimeAxis;
            }
        }

        /// <summary>
        /// Determines whether this tool can start.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this tool can start; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanStart()
        {
            IEnumerable<UIElement> elements = VisualTreeHelper.FindElementsInHostCoordinates(this.LastInput.ViewMousePosition, this.Timeline);

            foreach (UIElement element in elements)
            {
                EventPoint handledPoint = element as EventPoint;
                if (handledPoint != null)
                {
                    this.HandledPoint = handledPoint;
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Called when this tool is started and the mouse is moved.
        /// </summary>
        public override void MouseMove()
        {
            double value = GetWindowBasedPixelValue(this.LastInput.DocMousePosition.X);

            (this.HandledPoint.EventEntry as DateTimeEntry).Time = new DateTime((long)value);
        }

        /// <summary>
        /// Represents a tool that is used for scrolling the timeline.
        /// </summary>
        /// <summary>
        /// Called when this tool is started and the left mouse button is up.
        /// </summary>
        public override void MouseLeftButtonUp()
        {
            this.StopTool();
        }

    
        private double GetWindowBasedPixelValue(double pixelPosition)
        {
            double min = this.Axis.VisibleMinimum.Ticks;
            double max = this.Axis.VisibleMaximum.Ticks;

            double valuePosition = pixelPosition * (max - min) / (this.Axis.Width * this.Axis.ScrollScale) + min;

            return valuePosition;
        }
    }
}
